import { DomHelper, DragHelper, Rectangle, Scheduler } from '@bryntum/scheduler/scheduler.lite.umd.js';

export class MyDrag extends DragHelper {
    scheduler: Scheduler;

    static get defaultConfig() {
        return {
            // Don't drag the actual element, clone it
            cloneTarget: true,
            mode: 'translateXY',
            // Only allow drops on DOM elements with 'b-timeline-subgrid' CSS class specified
            dropTargetSelector: '.b-timeline-subgrid',
            // Only allow dragging elements with the 'draggable' CSS class
            targetSelector: '.draggable'
        };
    }

    constructor(config, scheduler) {
        super(config);
        this.scheduler = scheduler;

        this.on({
            dragstart: this.onDragStart,
            drop: this.onDrop
        });
    }

    onDragStart = function({ event, context }) {
        // Here you identify what you are dragging (an image of a user, grid row in an order table etc) and map it to something in your
        // data model. You can store your data on the context object which is available to you in all drag-related events
        context.eventName = context.grabbed.innerText;
    };

    onDrop = function({ source, context, event }) {
        if (context.valid) {
            const
                rect = Rectangle.client(context.element, DomHelper.up(context.target, source.config.dropTargetSelector)),
                startDate = this.scheduler.getDateFromCoordinate(rect.x, 'round', true, true),
                endDate = this.scheduler.getDateFromCoordinate(rect.right, 'round', true, true),
                resource = this.scheduler.resolveResourceRecord(event);

            this.scheduler.eventStore.add({
                name: context.eventName,
                resourceId: resource.id,
                startDate,
                endDate
            });

            // tell the drag helper the operation is finished
            context.finalize();
        }
    };

    setScheduler(scheduler: Scheduler) {
        this.scheduler = scheduler;
    }
}
